""":term:`NREN` L3 Core Service product type."""

from orchestrator.domain import SubscriptionModel
from orchestrator.types import SubscriptionLifecycle
from pydantic_forms.types import strEnum

from gso.products.product_blocks.nren_l3_core_service import (
    NRENL3CoreServiceBlock,
    NRENL3CoreServiceBlockInactive,
    NRENL3CoreServiceBlockProvisioning,
)


class NRENL3CoreServiceType(strEnum):
    """Available types of :term:`NREN` Layer 3 Core Services.

    The core services offered include GÉANT IP for R&E access, and the Internet Access Service.
    """

    GEANT_IP = "GÉANT IP"
    IMPORTED_GEANT_IP = "IMPORTED GÉANT IP"
    IAS = "IAS"
    IMPORTED_IAS = "IMPORTED IAS"


class NRENL3CoreServiceInactive(SubscriptionModel, is_base=True):
    """An inactive :term:`NREN` L3 Core Service subscription."""

    nren_l3_core_service_type: NRENL3CoreServiceType
    nren_l3_core_service: NRENL3CoreServiceBlockInactive


class NRENL3CoreServiceProvisioning(NRENL3CoreServiceInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A :term:`NREN` L3 Core Service subscription that's being provisioned."""

    nren_l3_core_service_type: NRENL3CoreServiceType
    nren_l3_core_service: NRENL3CoreServiceBlockProvisioning


class NRENL3CoreService(NRENL3CoreServiceProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An active :term:`NREN` L3 Core Service subscription."""

    nren_l3_core_service_type: NRENL3CoreServiceType
    nren_l3_core_service: NRENL3CoreServiceBlock


class ImportedNRENL3CoreServiceInactive(SubscriptionModel, is_base=True):
    """An imported, inactive :term:`NREN` L3 Core Service subscription."""

    nren_l3_core_service_type: NRENL3CoreServiceType
    nren_l3_core_service: NRENL3CoreServiceBlockInactive


class ImportedNRENL3CoreService(
    ImportedNRENL3CoreServiceInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING, SubscriptionLifecycle.ACTIVE]
):
    """An imported :term:`NREN` L3 Core Service subscription."""

    nren_l3_core_service_type: NRENL3CoreServiceType
    nren_l3_core_service: NRENL3CoreServiceBlock
